defmodule Pleroma.Mfc.Notify do
  @type task :: :user_change | :account_creation | :sync_follows | :status_create | :status_delete

  @spec perform(task(), User.t() | Activity.t()) :: any()
  def perform(:user_change, user) do
    Pleroma.Web.Mfc.Api.notify_user_change(user)
  end

  def perform(:account_creation, user) do
    Pleroma.Web.Mfc.Api.notify_account_creation(user)
  end

  def perform(:sync_follows, user) do
    Pleroma.Web.Mfc.Utils.sync_follows(user)
  end

  def perform(:status_create, activity) do
    Pleroma.Web.Mfc.Api.notify_status_creation(activity)
  end

  @spec perform(task(), Activity.t(), User.t(), NaiveDateTime.t()) :: any()
  def perform(:status_delete, activity, user, inserted_at) do
    Pleroma.Web.Mfc.Api.notify_status_deletion(activity, user, inserted_at)
  end
end
