# Pleroma: A lightweight social networking server
# Copyright © 2017-2019 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule Pleroma.Web.MongooseIM.MongooseIMController do
  use Pleroma.Web, :controller
  alias Pleroma.Web.Auth.MongooseIMAuthenticator

  def user_exists(conn, %{"user" => username}) do
    if MongooseIMAuthenticator.user_exists?(username) do
      conn
      |> json(true)
    else
      conn
      |> put_status(:not_found)
      |> json(false)
    end
  end

  def check_password(conn, %{"user" => username, "pass" => password}) do
    with true <-
           MongooseIMAuthenticator.check_password(%{
             username: username,
             password: password,
             ip: to_string(:inet.ntoa(conn.remote_ip))
           }) do
      conn
      |> json(true)
    else
      false ->
        conn
        |> put_status(:forbidden)
        |> json(false)

      _ ->
        conn
        |> put_status(:not_found)
        |> json(false)
    end
  end
end
