defmodule Pleroma.Web.RichMedia.Parser.TTL.MFCSignedUrl do
  @behaviour Pleroma.Web.RichMedia.Parser.TTL

  @impl Pleroma.Web.RichMedia.Parser.TTL
  def ttl(data, _url) do
    image_url = Map.get(data, :image)

    if is_mfc_signed_url(image_url) do
      image_url
      |> get_expiration_timestamp()
    end
  end

  defp is_mfc_signed_url(""), do: nil
  defp is_mfc_signed_url(nil), do: nil

  defp is_mfc_signed_url(url) when is_binary(url) do
    %URI{host: host} = URI.parse(url)
    String.contains?(host, "myfreecams.com")
  end

  defp is_mfc_signed_url(_), do: nil

  def get_expiration_timestamp(image_url) do
    list = Regex.run(~r/\/\d{10}\//, image_url)

    if is_list(list) and Enum.count(list) > 0 do
      list
      |> Enum.at(0)
      |> String.replace("/", "")
      |> String.to_integer()
    end
  end
end
