defmodule Pleroma.Web.Auth.MfcAuthenticatorTest do
  use Pleroma.DataCase, async: true
  import Pleroma.Factory
  alias Pleroma.Web.Auth.MfcAuthenticator

  describe "user_exists?/1" do
    test "returs true on exising mfc user" do
      user = insert(:user, mfc_id: "1")

      assert MfcAuthenticator.user_exists?(user.nickname)
    end

    test "returns false on non mfc user" do
      user = insert(:user)
      refute MfcAuthenticator.user_exists?(user.nickname)
    end
  end

  describe "check_password?/1" do
    test "returns true on existing user" do
      passcode_url =
        Application.get_env(:pleroma, :mfc)
        |> Keyword.get(:passcode_cookie_endpoint)

      login_url =
        Application.get_env(:pleroma, :mfc)
        |> Keyword.get(:login_endpoint)

      Tesla.Mock.mock(fn
        %{method: :post, url: ^passcode_url} ->
          %Tesla.Env{
            status: 200,
            headers: [
              {"set-cookie",
               "passcode=3nKHVtb3QDkNgKt3wsSnuak38vkHM5I6; expires=Thu, 28-Oct-2038 13:12:01 GMT; path=/; domain=myfreecams.com"},
              {"set-cookie",
               "username=lain; expires=Thu, 28-Oct-2038 13:14:27 GMT; path=/; domain=myfreecams.com"}
            ]
          }

        %{method: :post, url: ^login_url} ->
          %Tesla.Env{
            body:
              "{\"id\":\"0 0\",\"responseVer\":1,\"method\":\"internal\\/login\\/passcode\",\"result\":{\"user_id\":3004379,\"username\":\"testcam20\",\"access_level\":1,\"avatar_url\":false},\"err\":0}",
            status: 200
          }
      end)

      assert MfcAuthenticator.check_password(%{
               username: "lain",
               password: "wired",
               ip: "1.2.3.4"
             })
    end

    test "returns false if login was unsuccessful" do
      passcode_url =
        Application.get_env(:pleroma, :mfc)
        |> Keyword.get(:passcode_cookie_endpoint)

      Tesla.Mock.mock(fn
        %{method: :post, url: ^passcode_url} ->
          %Tesla.Env{
            status: 403,
            headers: []
          }
      end)

      refute MfcAuthenticator.check_password(%{
               username: "lain",
               password: "password",
               ip: "1.2.3.4"
             })
    end
  end
end
