defmodule Pleroma.Web.Auth.PleromaAuthenticatorTest do
  use Pleroma.DataCase, async: true
  import Pleroma.Factory
  alias Pleroma.Web.Auth.PleromaAuthenticator

  describe "user_exists?/1" do
    test "returns false for non existing user" do
      refute PleromaAuthenticator.user_exists?("some nickname")
    end

    test "returns true on existing user" do
      user = insert(:user)
      assert PleromaAuthenticator.user_exists?(user.nickname)
    end
  end

  describe "check_password/1" do
    test "not correct password returns false" do
      user = insert(:user)

      refute PleromaAuthenticator.check_password(%{
               username: user.nickname,
               password: "not right pwd"
             })
    end

    test "correct password returns true" do
      user = insert(:user, password_hash: Comeonin.Pbkdf2.hashpwsalt("good_password"))

      assert PleromaAuthenticator.check_password(%{
               username: user.nickname,
               password: "good_password"
             })
    end
  end
end
